/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "D22SpectroSampleSequencer.h"
#include "controllers/common/utilities/DataParams.h"
#include "controllers/common/family/Families.h"

#include "LoggingServices/Level.h"
#include "LoggingServices/Trace.h"

namespace d22special {

const std::string D22SpectroSampleSequencer::TYPE = "d22spectroSampleSequencer";
const int32 D22SpectroSampleSequencer::MAX_POSITIONS = 25;

const char D22SpectroSampleSequencer::DATAPARAM_THICKNESS_PROPERTY[] = "dataparam_thickness";
const char D22SpectroSampleSequencer::DATAPARAM_TRANSMISSION_PROPERTY[] = "dataparam_transmission";

D22SpectroSampleSequencer::D22SpectroSampleSequencer(const std::string & name) :
		ExperimentController(name), controller::Stoppable(this) {

	setFamily(family::SAMPLE_ENVIRONMENT);

	m_dataParams.init(this, "DataParams");
	m_sampleChanger.init(this, "SampleChanger");
	countWithSpectro.init(this, "count_spectro");

	maxPos.init(this, SAVE, "max_pos");

	maxSlot.init(this, SAVE, "max_slot");
	numSlot.init(this, SAVE, "actual_numSlot", "wanted_numSlot", "Num Slot");

	changerNum.init(this, SAVE, "changer_num");
	changer.init(this, NOSAVE, "actual_changer", "wanted_changer", "Changer");
	changerValues.init(this, NOSAVE, "changer_values");
	changerLabels.init(this, NOSAVE, "changer_labels");

	samplePosition.init(this, NOSAVE, "sample_position");
	acquisitionTime.init(this, NOSAVE, "acquisition_time");
	transmission.init(this, NOSAVE, "transmission");
	thickness.init(this, NOSAVE, "thickness");
	acquisitionTitle.init(this, NOSAVE, "acquisition_title");

	dataparamsThickness.init(this, SAVE, "dataparam_thickness");
	dataparamsTransmission.init(this, SAVE, "dataparam_transmission");

	timeType.init(this, SAVE, "time_type");
	integrationTime.init(this, SAVE, "integration_time"); //in microseconds!
	numSpectroMeasures.init(this, SAVE, "num_spectro");
	scansToAverage.init(this, SAVE, "scans_average");

	totalTime.init(this, SAVE, "total_time");

	//default initialization
	m_ChangerStarted = false;
	m_CountStarted = false;

	maxPos = MAX_POSITIONS;

	samplePosition.resize(MAX_POSITIONS);
	acquisitionTime.resize(MAX_POSITIONS);
	transmission.resize(MAX_POSITIONS);
	thickness.resize(MAX_POSITIONS);
	acquisitionTitle.resize(MAX_POSITIONS);

	for (int i = 0; i < MAX_POSITIONS; i++) {
		//fill new properties with default values
		int32 index = i + 1;
		ostringstream title;
		title << "Sample " << index;

		samplePosition.set(i, index);
		acquisitionTime.set(i, 10.0);
		transmission.set(i, 1.0);
		thickness.set(i, 0.1);
		acquisitionTitle.set(i, title.str());
	}

}

D22SpectroSampleSequencer::~D22SpectroSampleSequencer() {

}


D22SpectroSampleSequencer::D22SpectroSampleSequencer(const D22SpectroSampleSequencer& controller) :
	ExperimentController(controller),
	controller::Stoppable(this) {

	maxPos.copy(this, controller.maxPos);

	maxSlot.copy(this, controller.maxSlot);
	numSlot.copy(this, controller.numSlot);
	numSlot.setpoint.copy(this, controller.numSlot.setpoint);

	changerNum.copy(this, controller.changerNum);
	changer.copy(this, controller.changer);
	changer.setpoint.copy(this, controller.changer.setpoint);
	changerValues.copy(this, controller.changerValues);
	changerLabels.copy(this, controller.changerLabels);

	dataparamsThickness.copy(this, controller.dataparamsThickness);
	dataparamsTransmission.copy(this, controller.dataparamsTransmission);

	samplePosition.copy(this, controller.samplePosition);
	acquisitionTime.copy(this, controller.acquisitionTime);
	transmission.copy(this, controller.transmission);
	thickness.copy(this, controller.thickness);
	acquisitionTitle.copy(this, controller.acquisitionTitle);

	timeType.copy(this, controller.timeType);
	integrationTime.copy(this, controller.integrationTime); //in microseconds!
	numSpectroMeasures.copy(this, controller.numSpectroMeasures);
	scansToAverage.copy(this, controller.scansToAverage);

	totalTime.copy(this, controller.totalTime);

	//default initialization
	m_ChangerStarted = false;
	m_CountStarted = false;

	countWithSpectro.copy(controller.countWithSpectro);
	m_sampleChanger.copy(controller.m_sampleChanger);
	m_dataParams.copy(controller.m_dataParams);

}

void D22SpectroSampleSequencer::postConfiguration() {

	Publisher::attach(this);

	//Init values from sampleChanger
	int32 size = 8; //we use old sampleChanger controller whose size is fixed to 8
	for (int i = 0; i < size; i++) {
		changerNum.set(size);
		changerValues.resize(size);
		changerLabels.resize(size);
		for (int i = 0; i < size; i++) {
			//the same thing... very careful with this, we fill the combo with "static" values from sampleChanger (that is, values defined in its .properties file!)
			int32 index = i + 1;
			ostringstream title;
			title << "Changer# " << index;
			int32 value = index;
			std::string label = title.str(); //Changer#i
			changerValues.set(i, value);
			changerLabels.set(i, label);
		}
		changer.setpoint.setEnumeratedValues(changerValues);
		changer.setpoint.setEnumeratedLabels(changerLabels);
	}
	//Another default values
	maxSlot = m_sampleChanger->currentSlotMaxNum();    //any_cast<int32>(m_sampleChanger->getValue(SampleChangerController::MAX_CHA_PROPERTY));
	changer.setpoint = 1;
	totalTime = 10;

	registerRefresher(scansToAverage, &ExperimentController::copyValue<int32>, this, countWithSpectro->scansToAverage);
	registerRefresher(numSpectroMeasures, &ExperimentController::copyValue<int32>, this, countWithSpectro->numSpectroMeasures);
	registerRefresher(integrationTime, &ExperimentController::copyValue<int32>, this, countWithSpectro->integrationTime);
	registerRefresher(changer.setpoint, &D22SpectroSampleSequencer::refreshChangerSetpoint, this);
	registerRefresher(numSlot.setpoint, &D22SpectroSampleSequencer::refreshNumSlotSetpoint, this);

	registerRefresher(acquisitionTime, &D22SpectroSampleSequencer::refreshAcquisitionTime, this);

}

void D22SpectroSampleSequencer::refreshChangerSetpoint(int32 number) {
	m_sampleChanger->changer = number;
	maxSlot = m_sampleChanger->currentSlotMaxNum();
//	m_sampleChanger->setValue(SampleChangerController::CHANGER_NUMBER_PROPERTY, number);
//	maxSlot = any_cast<int32>(m_sampleChanger->getValue(SampleChangerController::MAX_CHA_PROPERTY));
}

void D22SpectroSampleSequencer::refreshNumSlotSetpoint(int32 number) {
	totalTime = 0;
	for (int32 i = 0; i < number; ++i) {
		totalTime = totalTime() + acquisitionTime.get(i);
	}
}

void D22SpectroSampleSequencer::refreshAcquisitionTime(int32 index, float64 value) throw (CannotSetValue) {
	//Calculate total time
	totalTime = 0;
	for (int32 i = 0; i < numSlot.setpoint(); ++i) {
		if (i == index) {
			totalTime = totalTime() + value; //done like this, otherwise when doing a set of the property the server crashes...

		} else {
			totalTime = totalTime() + acquisitionTime.get(i);
		}
	}
}

void D22SpectroSampleSequencer::start() {

	bool countSpectroControllerOk = true;

	int32 apply_nbSlot = numSlot.setpoint();
	float64 apply_acquisitionTime[apply_nbSlot];
	int32 apply_samplePosition[apply_nbSlot];
	float64 apply_Transmission[apply_nbSlot];
	float64 apply_Thickness[apply_nbSlot];
	string apply_acquisitionTitle[apply_nbSlot];

	for (int32 i = 0; i < apply_nbSlot; i++) {
		apply_samplePosition[i] = samplePosition.get(i);
		apply_acquisitionTime[i] = acquisitionTime.get(i);
		apply_Transmission[i] = transmission.get(i);
		apply_Thickness[i] = thickness.get(i);
		apply_acquisitionTitle[i] = acquisitionTitle.get(i);
	}

	commandProgression = 0;
	commandStatus.setRunning();

	for (int32 i = 0; i < apply_nbSlot; ++i) {
		try {
			if (apply_acquisitionTime[i] != 0.0) {

				commandProgression = ((float64) i / (float64) numSlot.setpoint()) * 100;

				//(1) Set up values of controllers
				m_sampleChanger->slot.setpoint = apply_samplePosition[i];
//				m_sampleChanger->setValue(SampleChangerController::WANTED_CHA_PROPERTY, apply_samplePosition[i]);

				try {
					//We will always work with the count controller in TIME MODE
					countWithSpectro->countTime = apply_acquisitionTime[i]; //units?
					countWithSpectro->subtitle = apply_acquisitionTitle[i];
					countWithSpectro->timeType = timeType();

					// Set total repetition to 1 by default (TODO)
//						m_Count->setValue(Count::TOTAL_REPS_PROPERTY, (int32 ) 1);
					//
				} catch (CannotSetValue) {
					log(Level::s_Error) << name << emptycursor << "Cannot set value for acquisition controller" << endlog;
					countSpectroControllerOk = false;
				}

				try {
					// Set values for transmission and thickness
//					m_dataParams->setValue(any_cast<string>(m_dataParams->getValue(DATAPARAM_TRANSMISSION_PROPERTY)), apply_Transmission[i]); //TODO check it is correct
//					m_dataParams->setValue("Thickness", apply_Thickness[i]); //TODO check it is correct
				} catch (...) {
					cerr << "error in setting dataParams properties" << endl;
				}

				// (2) Move sampleChanger
				m_ChangerStarted = true;
				m_sampleChanger->startCommand();
//				Sender<ClientCommand>::begin();
//				Sender<ClientCommand>::post(ClientCommand(m_sampleChanger, START_COMMAND));
//				Sender<ClientCommand>::endIf();
				m_ChangerStarted = false;

				if (commandStatus.isWarning()) {
					//the state warning means the controller has been stopped during its execution
					break;
				}

				commandProgression = commandProgression() + (100 / numSlot.setpoint() * 0.5);

				// (3) Move CountWithSpectro
				if (countSpectroControllerOk) {
					m_CountStarted = true;
					countWithSpectro->useSwitcher = false; //always in NO switcher mode
					countWithSpectro->startCommand();
					m_CountStarted = false;
				} else {
					log(Level::s_Error) << name << emptycursor << "Cannot start acquisition controller" << endlog;
				}
				//TODO: log here

				if (commandStatus.isWarning()) {
					break;
				}
			}
		} catch (CannotSetValue &e) {
			cerr << "Cannot set value" << endl;
		}
	}

	commandStatus.setIdle();
}

void D22SpectroSampleSequencer::stop() {
	if (commandStatus.isRunning()) {
		if (m_CountStarted) {
			countWithSpectro->stopCommand();
			m_CountStarted = false;
		} else if (m_ChangerStarted) {
			m_sampleChanger->stopCommand();
//			Sender<ClientCommand>::post(ClientCommand(m_sampleChanger, STOP_COMMAND));
			m_ChangerStarted = false;
		}
	}
//	log((logging ? Level::s_Info : Level::s_Debug)) << name << emptycursor << "stopped" << endlog;
	commandStatus.setWarning();

}

/*
 * setDisabled
 */
void D22SpectroSampleSequencer::setDisabled(bool all) {
	addExcludeSetEnableController(m_sampleChanger.getRole());
	ExperimentController::setDisabled(all);
}

}

