/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include <cmath>
#include <bitset>

#include "drivers/stereo_elec/StereoElecDef.h"

#include "DAQParamController.h"
#include "controllers/common/family/Families.h"

using namespace std;

namespace stereo_daq {

const string DAQParamController::TYPE = "daqparam_controller";

/*
 * Constructor
 */
DAQParamController::DAQParamController(const string& name) :
acquisition::TimeAcquisitionController(name), Select(this),Set(this) {

	setFamily(family::ACQUISITION,family::SETTING);

	// Init property
	daqmode.init(this, NOSAVE, "daqmode");
	trigmode.init(this, NOSAVE, "trigmode");
	reqcoinc.init(this, NOSAVE, "reqcoinc");
	nsample.init(this, NOSAVE, "nsample");
	fileformat.init(this, NOSAVE, "fileformat");
	filtercoef.init(this, NOSAVE, "filtercoef");
	ledused.init(this, NOSAVE, "ledused");
	nfeboardmax.init(this, NOSAVE, "nfeboardmax");
	nfeboard.init(this, NOSAVE, "nfeboard");
	ntot.init(this, NOSAVE, "ntot");
	ntail.init(this, NOSAVE, "ntail");
	pretrig.init(this, NOSAVE, "pretrig");
	cfddelay.init(this, NOSAVE, "cfddelay");
	cfdfraction.init(this, NOSAVE, "cfdfraction");
	precfd.init(this, NOSAVE, "precfd");
	trigchannel.init(this, NOSAVE, "trigchannel");
	trigsum03.init(this, NOSAVE, "trigsum03");
	trigsum47.init(this, NOSAVE, "trigsum47");
	trigsum07.init(this, NOSAVE, "trigsum07");
	trigquantity.init(this, NOSAVE, "trigquantity");
	slotindex.init(this, NOSAVE, "slotindex");
	sumthres.init(this, NOSAVE, "sumthres");
	nsum.init(this, NOSAVE, "nsum");
	t1accepted.init(this, NOSAVE, "t1accepted");
	t1timereset.init(this, NOSAVE, "t1timereset");
//	enabledfe.init(this, NOSAVE, "enabledfe");

	ntotchannel.init(this, NOSAVE, "ntotchannel");
	islot.init(this, NOSAVE, "islot");
	ichannel.init(this, NOSAVE, "ichannel");
	namechannel.init(this, NOSAVE, "namechannel");
	gain20.init(this, NOSAVE, "gain20");
	trigthres.init(this, NOSAVE, "trigthres");
	cfdthres.init(this, NOSAVE, "cfdthres");
	isselected.init(this, NOSAVE, "isselected");

	selectchoice.init(this,NOSAVE,"selectchoice");
	selectslot.init(this,NOSAVE,"selectslot");
	selectstring.init(this,NOSAVE,"selectstring");
	setparam.init(this, NOSAVE, "setparam");
	setvalue.init(this, NOSAVE, "setvalue");

	trigthresall.init(this, NOSAVE, "trigthresall");

	ntotall.init(this, NOSAVE, "ntotall");
	ntailall.init(this, NOSAVE, "ntailall");

	t2tgon.init(this, NOSAVE, "t2tgon");
	t2gcon.init(this, NOSAVE, "t2gcon");
	t2vton.init(this, NOSAVE, "t2vton");
	t2tgor.init(this, NOSAVE, "t2tgor");
	t2gcor.init(this, NOSAVE, "t2gcor");
	t2vtor.init(this, NOSAVE, "t2vtor");
	t2tgmode.init(this, NOSAVE, "t2tgmode");
	t2gcmode.init(this, NOSAVE, "t2gcmode");
	t2vtmode.init(this, NOSAVE, "t2vtmode");
	t2tgmin.init(this, NOSAVE, "t2tgmin");
	t2gcmin.init(this, NOSAVE, "t2gcmin");
	t2vtmin.init(this, NOSAVE, "t2vtmin");
	t2tgmax.init(this, NOSAVE, "t2tgmax");
	t2gcmax.init(this, NOSAVE, "t2gcmax");
	t2vtmax.init(this, NOSAVE, "t2vtmax");
	t2timeresetthres.init(this, NOSAVE, "t2timeresetthres");

	m_HVDriver.init(this, "hvdriver");

	experimentData.init(this, "experimentinfo");

	m_StartActivated = false;

	registerFunction(TYPE);
	registerRefresher(nfeboardmax,&DAQParamController::refreshNFEBoardMaxProperty,this);
	registerRefresher(nfeboard,&DAQParamController::refreshNFEBoardProperty,this);
	registerRefresher(daqmode, &DAQParamController::refreshDAQModeProperty, this);
	registerRefresher(trigmode, &DAQParamController::refreshTrigModeProperty, this);
	registerRefresher(reqcoinc, &DAQParamController::refreshReqCoincProperty, this);
	registerRefresher(nsample, &DAQParamController::refreshNSampleProperty, this);
	registerRefresher(fileformat, &DAQParamController::refreshFileFormatProperty, this);
	registerRefresher(filtercoef, &DAQParamController::refreshFilterCoefProperty, this);
	registerRefresher(ledused, &DAQParamController::refreshLedUsedProperty, this);
	registerRefresher(ntot, &DAQParamController::refreshNTotProperty, this);
	registerRefresher(ntail, &DAQParamController::refreshNTailProperty, this);
	registerRefresher(pretrig, &DAQParamController::refreshPretrigProperty, this);
	registerRefresher(trigquantity, &DAQParamController::refreshTrigQuantityProperty, this);
	registerRefresher(trigchannel, &DAQParamController::refreshTrigChannelProperty, this);
	registerRefresher(trigsum03, &DAQParamController::refreshTrigSum03Property, this);
	registerRefresher(trigsum47, &DAQParamController::refreshTrigSum47Property, this);
	registerRefresher(trigsum07, &DAQParamController::refreshTrigSum07Property, this);
	registerRefresher(cfddelay, &DAQParamController::refreshCFDDelayProperty, this);
	registerRefresher(cfdfraction, &DAQParamController::refreshCFDFractionProperty, this);
	registerRefresher(gain20, &DAQParamController::refreshGain20Property, this);
	registerRefresher(trigthres, &DAQParamController::refreshTrigThresProperty, this);
	registerRefresher(cfdthres, &DAQParamController::refreshCFDThresProperty, this);
	registerRefresher(sumthres, &DAQParamController::refreshSumThresProperty, this);
	registerRefresher(precfd, &DAQParamController::refreshPreCFDProperty, this);
	registerRefresher(nsum, &DAQParamController::refreshNSumProperty, this);
	registerRefresher(t1accepted, &DAQParamController::refreshT1AcceptedProperty, this);
	registerRefresher(t1timereset, &DAQParamController::refreshT1TimeResetProperty, this);
	registerRefresher(namechannel, &DAQParamController::refreshNameChannelProperty, this);
	registerRefresher(trigthresall, &DAQParamController::refreshTrigThresAllProperty, this);
	registerRefresher(ntotall, &DAQParamController::refreshNTotAllProperty, this);
	registerRefresher(ntailall, &DAQParamController::refreshNTailAllProperty, this);
	registerRefresher(t2tgon, &DAQParamController::refreshT2TGOnProperty, this);
	registerRefresher(t2gcon, &DAQParamController::refreshT2GCOnProperty, this);
	registerRefresher(t2vton, &DAQParamController::refreshT2VTOnProperty, this);
	registerRefresher(t2tgor, &DAQParamController::refreshT2TGOrProperty, this);
	registerRefresher(t2gcor, &DAQParamController::refreshT2GCOrProperty, this);
	registerRefresher(t2vtor, &DAQParamController::refreshT2VTOrProperty, this);
	registerRefresher(t2tgmode, &DAQParamController::refreshT2TGModeProperty, this);
	registerRefresher(t2gcmode, &DAQParamController::refreshT2GCModeProperty, this);
	registerRefresher(t2vtmode, &DAQParamController::refreshT2VTModeProperty, this);
	registerRefresher(t2tgmin, &DAQParamController::refreshT2TGMinProperty, this);
	registerRefresher(t2gcmin, &DAQParamController::refreshT2GCMinProperty, this);
	registerRefresher(t2vtmin, &DAQParamController::refreshT2VTMinProperty, this);
	registerRefresher(t2tgmax, &DAQParamController::refreshT2TGMaxProperty, this);
	registerRefresher(t2gcmax, &DAQParamController::refreshT2GCMaxProperty, this);
	registerRefresher(t2vtmax, &DAQParamController::refreshT2VTMaxProperty, this);
	registerRefresher(t2timeresetthres, &DAQParamController::refreshT2TimeResetThresProperty, this);


	//	registerRefresher(enabledfe, &DAQParamController::refreshEnabledFEProperty, this);


}

DAQParamController::DAQParamController(const DAQParamController& controller) :
		acquisition::TimeAcquisitionController(controller), Select(this),Set(this) {

	daqmode.copy(this, controller.daqmode);
	trigmode.copy(this, controller.trigmode);
	reqcoinc.copy(this, controller.reqcoinc);
	nsample.copy(this, controller.nsample);
	fileformat.copy(this, controller.fileformat);
	filtercoef.copy(this, controller.filtercoef);
	ledused.copy(this, controller.ledused);
	nfeboardmax.copy(this, controller.nfeboardmax);
	nfeboard.copy(this, controller.nfeboard);
	ntot.copy(this, controller.ntot);
	ntail.copy(this, controller.ntail);
	pretrig.copy(this, controller.pretrig);
	cfddelay.copy(this, controller.cfddelay);
	cfdfraction.copy(this, controller.cfdfraction);
	precfd.copy(this, controller.precfd);
	trigchannel.copy(this, controller.trigchannel);
	trigsum03.copy(this, controller.trigsum03);
	trigsum47.copy(this, controller.trigsum47);
	trigsum07.copy(this, controller.trigsum07);
	trigquantity.copy(this, controller.trigquantity);
	slotindex.copy(this, controller.slotindex);
	sumthres.copy(this, controller.sumthres);
	nsum.copy(this, controller.nsum);
	t1accepted.copy(this, controller.t1accepted);
	t1timereset.copy(this,controller.t1timereset);
//	enabledfe.copy(this,controller.enabledfe);

	ntotchannel.copy(this, controller.ntotchannel);
	islot.copy(this, controller.islot);
	ichannel.copy(this, controller.ichannel);
	namechannel.copy(this, controller.namechannel);
	gain20.copy(this, controller.gain20);
	trigthres.copy(this, controller.trigthres);
	cfdthres.copy(this, controller.cfdthres);
	isselected.copy(this, controller.isselected);

	selectchoice.copy(this, controller.selectchoice);
	selectslot.copy(this, controller.selectslot);
	selectstring.copy(this, controller.selectstring);
	setparam.copy(this, controller.setparam);
	setvalue.copy(this, controller.setvalue);

	trigthresall.copy(this, controller.trigthresall);
	ntotall.copy(this, controller.ntotall);
	ntailall.copy(this, controller.ntailall);

	t2tgon.copy(this, controller.t2tgon);
	t2gcon.copy(this, controller.t2gcon);
	t2vton.copy(this, controller.t2vton);
	t2tgor.copy(this, controller.t2tgor);
	t2gcor.copy(this, controller.t2gcor);
	t2vtor.copy(this, controller.t2vtor);
	t2tgmode.copy(this, controller.t2tgmode);
	t2gcmode.copy(this, controller.t2gcmode);
	t2vtmode.copy(this, controller.t2vtmode);
	t2tgmin.copy(this, controller.t2tgmin);
	t2gcmin.copy(this, controller.t2gcmin);
	t2vtmin.copy(this, controller.t2vtmin);
	t2tgmax.copy(this, controller.t2tgmax);
	t2gcmax.copy(this, controller.t2gcmax);
	t2vtmax.copy(this, controller.t2vtmax);
	t2timeresetthres.copy(this, controller.t2timeresetthres);

	m_HVDriver.copy(controller.m_HVDriver);
	experimentData.copy(controller.experimentData);

	m_StartActivated = false;

	registerRefresher(nfeboard,&DAQParamController::refreshNFEBoardCloneProperty,this);
	registerRefresher(nfeboardmax,&DAQParamController::refreshNFEBoardMaxProperty,this);
	registerRefresher(ntotall, &DAQParamController::refreshNTotAllProperty, this);
	registerRefresher(ntailall, &DAQParamController::refreshNTailAllProperty, this);
}

/*
 * Destructor
 */
DAQParamController::~DAQParamController() {
	// Empty
}

void DAQParamController::postConfiguration() {

	TimeAcquisitionController::postConfiguration();

	m_ELECDriver.copy(m_Drivers[0]);

	registerUpdater(m_ELECDriver->triggeripaddress, &DAQParamController::updateTriggerIPAddress, this);
	registerUpdater(m_ELECDriver->nfeboard, &DAQParamController::updateNFEBoard, this);
	registerUpdater(m_ELECDriver->boardipaddress, &DAQParamController::updateBoardIPAddress, this);
	registerUpdater(m_ELECDriver->namechannel0, &DAQParamController::updateNameChannel0, this);
	registerUpdater(m_ELECDriver->namechannel1, &DAQParamController::updateNameChannel1, this);
	registerUpdater(m_ELECDriver->namechannel2, &DAQParamController::updateNameChannel2, this);
	registerUpdater(m_ELECDriver->namechannel3, &DAQParamController::updateNameChannel3, this);
	registerUpdater(m_ELECDriver->namechannel4, &DAQParamController::updateNameChannel4, this);
	registerUpdater(m_ELECDriver->namechannel5, &DAQParamController::updateNameChannel5, this);
	registerUpdater(m_ELECDriver->namechannel6, &DAQParamController::updateNameChannel6, this);
	registerUpdater(m_ELECDriver->namechannel7, &DAQParamController::updateNameChannel7, this);
	registerUpdater(m_ELECDriver->startActivated, &DAQParamController::updateStartActivated, this);
	registerUpdater(m_ELECDriver->logstring, &DAQParamController::updateLogString, this);

	registerUpdater(experimentData->title, &DAQParamController::updateTitle, this);
	registerUpdater(experimentData->subtitle, &DAQParamController::updateSubtitle, this);


	nfeboardmax=stereo_elec::NBFEMAX;
	nfeboard = m_ELECDriver->nfeboard();
	m_ELECDriver->DAQParams.triggerIPAdd = m_ELECDriver->triggeripaddress();
	m_ELECDriver->DAQParams.isNatR5=false;
	daqmode = 1;
	trigmode = 0;
	reqcoinc = 0;
	nsample = 60;
	fileformat="ROOT";
	filtercoef=130928;
	ledused=0;
	m_ELECDriver->DAQParams.ledUsed=false;
	trigthresall=0;
	ntotall=0;
	ntailall=0;
	setparam="";
	setvalue="0";
	m_ELECDriver->DAQParams.sourceType="none";
	m_ELECDriver->DAQParams.sourcePosX=0.;
	m_ELECDriver->DAQParams.sourcePosY=0.;
	m_ELECDriver->DAQParams.sourcePosZ=0.;

	t2tgon=false;
	t2gcon=false;
	t2vton=false;
	t2timeresetthres=0;
	experimentData->title="";
	experimentData->subtitle="";
}

void DAQParamController::refreshNFEBoardMaxProperty(int32 value) throw (CannotSetValue) {\
	// Resize properties and vector
	ntot.resize(value);
	ntail.resize(value);
	pretrig.resize(value);
	cfddelay.resize(value);
	cfdfraction.resize(value);
	precfd.resize(value);
	trigchannel.resize(value);
	trigsum03.resize(value);
	trigsum47.resize(value);
	trigsum07.resize(value);
	trigquantity.resize(value);
	slotindex.resize(value);
	sumthres.resize(value);
	nsum.resize(value);
	t1accepted.resize(value);
	t1timereset.resize(value);
//	enabledfe.resize(value);


	islot.resize(value*stereo_elec::NBFECHANNEL);
	ichannel.resize(value*stereo_elec::NBFECHANNEL);
	namechannel.resize(value*stereo_elec::NBFECHANNEL);
	gain20.resize(value*stereo_elec::NBFECHANNEL);
	trigthres.resize(value*stereo_elec::NBFECHANNEL);
	cfdthres.resize(value*stereo_elec::NBFECHANNEL);
	isselected.resize(value*stereo_elec::NBFECHANNEL);

}

void DAQParamController::refreshNFEBoardProperty(int32 value) throw (CannotSetValue) {
	cout << std::showbase<<dec;
	cout << dec << "nfeboard ancienne " << nfeboard() <<" nouvelle " << value <<  endl;
	if(nfeboard()==value){
		cout << "nfeboard inchanged, skip the refresher" << endl ;
		return;
	}
	nfeboard.update(value);
	cout << dec << "nfeboard nouvelle " << nfeboard()  <<  endl;
//	// Resize properties and vector
//	ntot.resize(value);
//	ntail.resize(value);
//	pretrig.resize(value);
//	cfddelay.resize(value);
//	cfdfraction.resize(value);
//	precfd.resize(value);
//	trigchannel.resize(value);
//	trigsum03.resize(value);
//	trigsum47.resize(value);
//	trigsum07.resize(value);
//	trigquantity.resize(value);
//	slotindex.resize(value);
//	sumthres.resize(value);
//	nsum.resize(value);
//	t1accepted.resize(value);
//	t1timereset.resize(value);
//	enabledfe.resize(value);

	ntotchannel.set(value * stereo_elec::NBFECHANNEL);
	cout << dec<< "ntotchannel " << ntotchannel() << endl;
//	islot.resize(ntotchannel());
//	ichannel.resize(ntotchannel());
//	namechannel.resize(ntotchannel());
//	gain20.resize(ntotchannel());
//	trigthres.resize(ntotchannel());
//	cfdthres.resize(ntotchannel());
//	isselected.resize(ntotchannel());

	m_ELECDriver->DAQParams.elecMapping_v.resize(ntotchannel());
	m_ELECDriver->DAQParams.slotHV_v.resize(ntotchannel());
	m_ELECDriver->DAQParams.channelHV_v.resize(ntotchannel());
	m_ELECDriver->DAQParams.fe_trigger_mask=0;
	m_ELECDriver->DAQParams.fe_reset_mask=0;
	m_ELECDriver->DAQParams.FEParams_v.resize(value);

	for (int i = 0; i < value; i++) {
		m_ELECDriver->DAQParams.FEParams_v.at(i).gain_v.resize(stereo_elec::NBFECHANNEL);
		m_ELECDriver->DAQParams.FEParams_v.at(i).threshold_v.resize(stereo_elec::NBFECHANNEL);
		m_ELECDriver->DAQParams.FEParams_v.at(i).CFDThreshold_v.resize(stereo_elec::NBFECHANNEL);
	}

	// initialisation slotindex property, IP address
	int32 index = 0;

	//cout << dec <<  "nslot " << m_ELECDriver->nslot() << endl;
	for (int j = 0; j < m_ELECDriver->nslot(); j++) {
		//cout << "j " << j << "index " << index<< " " << m_ELECDriver->boardtype(j) << endl;
		if (m_ELECDriver->boardtype(j) == stereo_elec::FEBOARD and index<value) {
			m_ELECDriver->DAQParams.isFEBoard_v.at(j) = true;
			m_ELECDriver->DAQParams.fe_trigger_mask|=  (1 << j);
			//m_ELECDriver->DAQParams.isFEBoardEnabled_v.at(j) = true;
			m_ELECDriver->DAQParams.FEParams_v.at(index).FE8IPAdd=m_ELECDriver->boardipaddress(j);
			m_ELECDriver->DAQParams.FEParams_v.at(index).slot_num = j+1;
			m_ELECDriver->DAQParams.FEParams_v.at(index).trigMask = 255;
			slotindex.set(index, j);
			cout << "IP " << m_ELECDriver->DAQParams.FEParams_v.at(index).FE8IPAdd << endl;
			index++;

		}
		else if (m_ELECDriver->boardtype(j) == stereo_elec::LEDBOARD) {
			m_ELECDriver->DAQParams.isFEBoard_v.at(j) = false;
			//m_ELECDriver->DAQParams.isFEBoardEnabled_v.at(j) = false;
			m_ELECDriver->DAQParams.ledBoardInserted =true;
			m_ELECDriver->DAQParams.ledIPAdd=m_ELECDriver->boardipaddress(j);
		}
		else {
			m_ELECDriver->DAQParams.isFEBoard_v.at(j) = false;
			//m_ELECDriver->DAQParams.isFEBoardEnabled_v.at(j) = false;
		}
		cout << "IsFeboard " << j << " " << m_ELECDriver->DAQParams.isFEBoard_v.at(j) << endl;
	}


	// initialisation properties with default values, except for namechannel and ip address
	index = 0;
	for (int i = 0; i < value; i++) {
		ntot.set(i, 50);
		ntail.set(i, 40);
		pretrig.set(i, 10);
		cfddelay.set(i, 2);
		cfdfraction.set(i, 13107);
		precfd.set(i, 2);
		trigchannel.set(i, true);
		trigsum03.set(i, false);
		trigsum47.set(i, false);
		trigsum07.set(i, false);
		//cout << "init trigquantity  " << endl ;
		trigquantity.set(i, TrigAmp);
		sumthres.set(i, 1000);
		nsum.set(i, 50);
		t1accepted.set(i,true);
		t1timereset.set(i,false);
//		enabledfe.set(i,true);

		for (int k = 0; k < stereo_elec::NBFECHANNEL; k++) {
			//cout << "index " << index << endl;
			islot.set(index, slotindex(i));
			ichannel.set(index, k);
			switch (k) {
			case 0:
				namechannel.set(index, m_ELECDriver->namechannel0(slotindex(i)));
				break;
			case 1:
				namechannel.set(index, m_ELECDriver->namechannel1(slotindex(i)));
				break;
			case 2:
				namechannel.set(index, m_ELECDriver->namechannel2(slotindex(i)));
				break;
			case 3:
				namechannel.set(index, m_ELECDriver->namechannel3(slotindex(i)));
				break;
			case 4:
				namechannel.set(index, m_ELECDriver->namechannel4(slotindex(i)));
				break;
			case 5:
				namechannel.set(index, m_ELECDriver->namechannel5(slotindex(i)));
				break;
			case 6:
				namechannel.set(index, m_ELECDriver->namechannel6(slotindex(i)));
				break;
			case 7:
				namechannel.set(index, m_ELECDriver->namechannel7(slotindex(i)));
				break;
			}
			isselected.set(index, false);
			gain20.set(index, false);
			trigthres.set(index, 110);
			cfdthres.set(index, 100);

			index++;
		}
	}
}

void DAQParamController::refreshNFEBoardCloneProperty(int32 value) throw (CannotSetValue) {

//	// Resize properties and vector
//	ntot.resize(value);
//	ntail.resize(value);
//	pretrig.resize(value);
//	cfddelay.resize(value);
//	cfdfraction.resize(value);
//	precfd.resize(value);
//	trigchannel.resize(value);
//	trigsum03.resize(value);
//	trigsum47.resize(value);
//	trigsum07.resize(value);
//	trigquantity.resize(value);
//	slotindex.resize(value);
//	sumthres.resize(value);
//	nsum.resize(value);
//	t1accepted.resize(value);
//	t1timereset.resize(value);
////	enabledfe.resize(value);
//
	ntotchannel.set(value * stereo_elec::NBFECHANNEL);
	cout << dec << "ntotchannel " << ntotchannel() << endl;
//	islot.resize(ntotchannel());
//	ichannel.resize(ntotchannel());
//	namechannel.resize(ntotchannel());
//	gain20.resize(ntotchannel());
//	trigthres.resize(ntotchannel());
//	cfdthres.resize(ntotchannel());
//	isselected.resize(ntotchannel());



	// initialisation slotindex property, IP address
	int32 index = 0;



	// initialisation properties with default values, no value for namechannel (clone)
	index = 0;
	for (int i = 0; i < value; i++) {
		ntot.set(i, 50);
		ntail.set(i, 40);
		pretrig.set(i, 10);
		cfddelay.set(i, 2);
		cfdfraction.set(i, 13107);
		precfd.set(i, 2);
		trigchannel.set(i, true);
		trigsum03.set(i, false);
		trigsum47.set(i, false);
		trigsum07.set(i, false);
		trigquantity.set(i, TrigAmp);
		sumthres.set(i, 1000);
		nsum.set(i, 50);
		t1accepted.set(i,true);
		t1timereset.set(i,false);
//		enabledfe.set(i,true);
		slotindex.set(i,-1);
		for (int k = 0; k < stereo_elec::NBFECHANNEL; k++) {
			//cout << "index " << index << endl;
			islot.set(index, slotindex(i));
			ichannel.set(index, -1);
			isselected.set(index, false);
			gain20.set(index, false);
			trigthres.set(index, 120);
			cfdthres.set(index, 100);
			namechannel.set(index,"");

			index++;
		}
	}

}

void DAQParamController::refreshDAQModeProperty(int32 value) throw (CannotSetValue) {
	m_ELECDriver->DAQParams.DAQMode = (short unsigned) value;
	//cout << "DAQMode " << m_ELECDriver->DAQParams.DAQMode << endl;
}
void DAQParamController::refreshTrigModeProperty(int32 value) throw (CannotSetValue) {
	m_ELECDriver->DAQParams.trigMode = (short unsigned) value;
	if(value==2) reqcoinc=0;
}
void DAQParamController::refreshReqCoincProperty(int32 value) throw (CannotSetValue) {
	//cout << "reqcoinc " << reqcoinc() << " value " << value << endl;
	if(trigmode()==2) value=0;
	else m_ELECDriver->DAQParams.reqCoinc = (short unsigned) value;

}
void DAQParamController::refreshNSampleProperty(int32 value) throw (CannotSetValue) {
	m_ELECDriver->DAQParams.nSamples = (short unsigned) value;
}
void DAQParamController::refreshFileFormatProperty(const std::string& value) throw (CannotSetValue) {
	m_ELECDriver->DAQParams.file_format = value;
}
void DAQParamController::refreshFilterCoefProperty(int value) throw (CannotSetValue) {
	m_ELECDriver->DAQParams.afrac = value;
}
void DAQParamController::refreshLedUsedProperty(int value) throw (CannotSetValue) {
	m_ELECDriver->DAQParams.ledUsed = value;
}
void DAQParamController::refreshNTotProperty(int32 index, int32 value) throw (CannotSetValue) {
	if(index<nfeboard()&&ntotall()==0){
		m_ELECDriver->DAQParams.FEParams_v.at(index).nTot = (short unsigned) value;
		//cout << "NTot(" << index << ") = " << value << endl;
	}
}

void DAQParamController::refreshNTailProperty(int32 index, int32 value) throw (CannotSetValue) {
	if(index<nfeboard()&&ntailall()==0){
		m_ELECDriver->DAQParams.FEParams_v.at(index).nTail = (short unsigned) value;
		//cout << "NTail(" << index << ") = " << value << endl;
	}
}

void DAQParamController::refreshPretrigProperty(int32 index, int32 value) throw (CannotSetValue) {
	if(index<nfeboard()) m_ELECDriver->DAQParams.FEParams_v.at(index).preTrig = (short unsigned) value;
}

void DAQParamController::refreshTrigQuantityProperty(int32 index, const std::string& value) throw (CannotSetValue) {
	if(index<nfeboard()){
		if (value == TrigSum) m_ELECDriver->DAQParams.FEParams_v.at(index).isTriggerOnQ = true;
		else m_ELECDriver->DAQParams.FEParams_v.at(index).isTriggerOnQ = false;
	}
}

void DAQParamController::refreshTrigChannelProperty(int32 index, bool value) throw (CannotSetValue) {
	if(index<nfeboard()){
		if (value) m_ELECDriver->DAQParams.FEParams_v.at(index).trigMask |= 0x00FF; // mise à 1 des 8 premiers bits
		else m_ELECDriver->DAQParams.FEParams_v.at(index).trigMask &= 0xFF00; // mise à 0 des 8 premiers bits
		//cout << "trigMask " << (bitset<16> ) m_ELECDriver->DAQParams.FEParams_v.at(index).trigMask << endl;
	}
}

void DAQParamController::refreshTrigSum03Property(int32 index, bool value) throw (CannotSetValue) {
	if(index<nfeboard()){
		if (value) m_ELECDriver->DAQParams.FEParams_v.at(index).trigMask |= (short unsigned) (1u << 8); // mise à 1 du bit 8
		else m_ELECDriver->DAQParams.FEParams_v.at(index).trigMask &= (short unsigned) ~(1u << 8); // mise à 0 du bit 8
		//cout << "trigMask " << (bitset<16> ) m_ELECDriver->DAQParams.FEParams_v.at(index).trigMask << endl;
	}
}

void DAQParamController::refreshTrigSum47Property(int32 index, bool value) throw (CannotSetValue) {
	if(index<nfeboard()){
		if (value) m_ELECDriver->DAQParams.FEParams_v.at(index).trigMask |= (short unsigned) (1u << 9); // mise à 1 du bit 9
		else m_ELECDriver->DAQParams.FEParams_v.at(index).trigMask &= (short unsigned) ~(1u << 9); // mise à 0 du bit 9
		//cout << "trigMask " << (bitset<16> ) m_ELECDriver->DAQParams.FEParams_v.at(index).trigMask << endl;

	}
}

void DAQParamController::refreshTrigSum07Property(int32 index, bool value) throw (CannotSetValue) {
	if(index<nfeboard()){
		if (value) m_ELECDriver->DAQParams.FEParams_v.at(index).trigMask |= (short unsigned) (1u << 10); // mise à 1 du bit 10
		else m_ELECDriver->DAQParams.FEParams_v.at(index).trigMask &= (short unsigned) ~(1u << 10); // mise à 0 du bit 10
		//cout << "trigMask " << (bitset<16> ) m_ELECDriver->DAQParams.FEParams_v.at(index).trigMask << endl;
	}
}

void DAQParamController::refreshCFDDelayProperty(int32 index, int32 value) throw (CannotSetValue) {
	if(index<nfeboard()) m_ELECDriver->DAQParams.FEParams_v.at(index).CFDDelay = (short unsigned) value;
}

void DAQParamController::refreshCFDFractionProperty(int32 index, int32 value) throw (CannotSetValue) {
	if(index<nfeboard()) m_ELECDriver->DAQParams.FEParams_v.at(index).CFDFrac = (short unsigned) value;
}

void DAQParamController::refreshPreCFDProperty(int32 index, int32 value) throw (CannotSetValue) {
	if(index<nfeboard()) m_ELECDriver->DAQParams.FEParams_v.at(index).preCFD = (short unsigned) value;
}

void DAQParamController::refreshSumThresProperty(int32 index, int32 value) throw (CannotSetValue) {
	if(index<nfeboard() && trigthresall()==0) m_ELECDriver->DAQParams.FEParams_v.at(index).sumThreshold = value;
}
void DAQParamController::refreshNSumProperty(int32 index, int32 value) throw (CannotSetValue) {
	if(index<nfeboard())m_ELECDriver->DAQParams.FEParams_v.at(index).nSum = (short unsigned) value;
}

void DAQParamController::refreshT1AcceptedProperty(int32 index, bool value) throw (CannotSetValue) {
	if(index<nfeboard()){
		if(value) m_ELECDriver->DAQParams.fe_trigger_mask|=  (1 << slotindex(index));
		else m_ELECDriver->DAQParams.fe_trigger_mask &= ~(1<< slotindex(index));
		//cout << "fe_trigger_mask "<<  m_ELECDriver->DAQParams.fe_trigger_mask << " value=" << value << endl;
	}
}
void DAQParamController::refreshT1TimeResetProperty(int32 index, bool value) throw (CannotSetValue) {
	if(index<nfeboard()){
		if(value) m_ELECDriver->DAQParams.fe_reset_mask|=  (1 << slotindex(index));
		else m_ELECDriver->DAQParams.fe_reset_mask &=  ~(1<< slotindex(index));
		//cout << "fe_reset_mask "<<  m_ELECDriver->DAQParams.fe_reset_mask << " value=" << value << endl;
	}
}
//void DAQParamController::refreshEnabledFEProperty(int32 index, bool value) throw (CannotSetValue) {
//	m_ELECDriver->DAQParams.isFEBoardEnabled_v.at(slotindex(index)) = value;
//
//}

void DAQParamController::refreshGain20Property(int32 index, bool value) throw (CannotSetValue) {
	//cout << "index " << index << " slot " << index/stereo_elec::NBFECHANNEL << " channel " << ichannel(index) << " value " << value << endl;
	if(index<ntotchannel()) m_ELECDriver->DAQParams.FEParams_v.at(index / stereo_elec::NBFECHANNEL).gain_v[ichannel(index)] = value;
}

void DAQParamController::refreshTrigThresProperty(int32 index, int32 value) throw (CannotSetValue) {
	if(index<ntotchannel()&&trigthresall()==0){
		m_ELECDriver->DAQParams.FEParams_v.at(index / stereo_elec::NBFECHANNEL).threshold_v[ichannel(index)] = (short unsigned) value;
		//cout << "refreshTrigThresProperty " <<m_ELECDriver->DAQParams.FEParams_v.at(index / stereo_elec::NBFECHANNEL).threshold_v[ichannel(index)] << endl;
	}
}

void DAQParamController::refreshCFDThresProperty(int32 index, int32 value) throw (CannotSetValue) {
	if(index<ntotchannel()) m_ELECDriver->DAQParams.FEParams_v.at(index / stereo_elec::NBFECHANNEL).CFDThreshold_v[ichannel(index)] =
			(short unsigned) value;

}
void DAQParamController::refreshNameChannelProperty(int32 index, const std::string& value) throw (CannotSetValue) {
	if(index<ntotchannel()){
		m_ELECDriver->DAQParams.elecMapping_v.at(index) = value;
		m_ELECDriver->DAQParams.slotHV_v.at(index) = -1;
		m_ELECDriver->DAQParams.channelHV_v.at(index) = -1;

		int i = 0;
		if (m_HVDriver->ntotchannel() > 0) {
			while (m_HVDriver->namechannel(i) != value && i < m_HVDriver->ntotchannel() - 1) {
				i++;
			}
			if (i < m_HVDriver->ntotchannel()) {
				m_ELECDriver->DAQParams.slotHV_v.at(index) = m_HVDriver->islot(i);
				m_ELECDriver->DAQParams.channelHV_v.at(index) = m_HVDriver->ichannel(i);
			}
		}
	}
}
void DAQParamController::refreshTrigThresAllProperty(int32 value) throw (CannotSetValue) {
	if(value>0){
		for(int i=0;i<nfeboard();i++){
			for(int j=0;j< stereo_elec::NBFECHANNEL; j++){
				m_ELECDriver->DAQParams.FEParams_v.at(i).threshold_v[j] = (short unsigned) value;
			}
			m_ELECDriver->DAQParams.FEParams_v.at(i).sumThreshold= (short unsigned) value;
		}

		log(Level::s_Info) << "Trigger threshold have been set to " << value << " for all channels" << endlog;
	}
}
void DAQParamController::refreshNTotAllProperty(int32 value) throw (CannotSetValue) {
	if(value>0){
		for(int i=0;i<nfeboard();i++){
			//m_ELECDriver->DAQParams.FEParams_v.at(i).nTot = (short unsigned) value;
			ntot.set(i,value);
		}
		log(Level::s_Info) << "Ntot have been set to " << value << " for all channels" << endlog;

	}
}
void DAQParamController::refreshNTailAllProperty(int32 value) throw (CannotSetValue) {
	if(value>0){
		for(int i=0;i<nfeboard();i++){
			//m_ELECDriver->DAQParams.FEParams_v.at(i).nTail = (short unsigned) value;
			ntail.set(i,value);
		}
		log(Level::s_Info) << "Ntail have been set to " << value << " for all channels" << endlog;

	}
}
void DAQParamController::refreshT2TGOnProperty(bool value) throw (CannotSetValue){
	if(value==false){
		t2tgmode=false;
		t2tgmin=0x80000000;
		t2tgmax=0x7FFFFFFF;
	}
}
void DAQParamController::refreshT2GCOnProperty(bool value) throw (CannotSetValue){
	if(value==false){
		t2gcmode=false;
		t2gcmin=0x80000000;
		t2gcmax=0x7FFFFFFF;
	}
}
void DAQParamController::refreshT2VTOnProperty(bool value) throw (CannotSetValue){
	if(value==false){
		t2vtmode=false;
		t2vtmin=0x80000000;
		t2vtmax=0x7FFFFFFF;
	}
}void DAQParamController::refreshT2TGOrProperty(bool value) throw (CannotSetValue){
	m_ELECDriver->DAQParams.OR_TG=value;
}
void DAQParamController::refreshT2GCOrProperty(bool value) throw (CannotSetValue){
	m_ELECDriver->DAQParams.OR_GC=value;
}
void DAQParamController::refreshT2VTOrProperty(bool value) throw (CannotSetValue){
	m_ELECDriver->DAQParams.OR_VT=value;
}
void DAQParamController::refreshT2TGModeProperty(bool value) throw (CannotSetValue){
	m_ELECDriver->DAQParams.TGmode=value;
}
void DAQParamController::refreshT2GCModeProperty(bool value) throw (CannotSetValue){
	m_ELECDriver->DAQParams.GCmode=value;
}
void DAQParamController::refreshT2VTModeProperty(bool value) throw (CannotSetValue){
	m_ELECDriver->DAQParams.VTmode=value;
}
void DAQParamController::refreshT2TGMinProperty(int32 value) throw (CannotSetValue){
	m_ELECDriver->DAQParams.QtotThreshold[0] = value;
}
void DAQParamController::refreshT2GCMinProperty(int32 value) throw (CannotSetValue){
	m_ELECDriver->DAQParams.QtotThreshold[2] = value;
}
void DAQParamController::refreshT2VTMinProperty(int32 value) throw (CannotSetValue){
	m_ELECDriver->DAQParams.QtotThreshold[4] = value;
}
void DAQParamController::refreshT2TGMaxProperty(int32 value) throw (CannotSetValue){
	m_ELECDriver->DAQParams.QtotThreshold[1] = value;
}
void DAQParamController::refreshT2GCMaxProperty(int32 value) throw (CannotSetValue){
	m_ELECDriver->DAQParams.QtotThreshold[3] = value;
}
void DAQParamController::refreshT2VTMaxProperty(int32 value) throw (CannotSetValue){
	m_ELECDriver->DAQParams.QtotThreshold[5] = value;
}
void DAQParamController::refreshT2TimeResetThresProperty(int32 value) throw (CannotSetValue){
	m_ELECDriver->DAQParams.QtotThreshold[6] = value;
}

void DAQParamController::updateTriggerIPAddress() {
	m_ELECDriver->DAQParams.triggerIPAdd = m_ELECDriver->triggeripaddress();
}

void DAQParamController::updateNFEBoard() {
	log(Level::s_Info) << "Number of FE boards changed from " << nfeboard() << " to " << m_ELECDriver->nfeboard() << endlog;
	nfeboard = m_ELECDriver->nfeboard();
}


void DAQParamController::updateBoardIPAddress(int32 index) {
	if (m_ELECDriver->boardtype(index) == stereo_elec::FEBOARD) {
		m_ELECDriver->DAQParams.FEParams_v.at(m_ELECDriver->feindex.at(index)).FE8IPAdd=m_ELECDriver->boardipaddress(index);
	}
	if (m_ELECDriver->boardtype(index) == stereo_elec::LEDBOARD) {
		m_ELECDriver->DAQParams.ledIPAdd=m_ELECDriver->boardipaddress(index);
	}
}

void DAQParamController::updateNameChannel0(int32 index) {
	if (m_ELECDriver->boardtype(index) == stereo_elec::FEBOARD) {
		namechannel.set(m_ELECDriver->feindex.at(index) * stereo_elec::NBFECHANNEL + 0, m_ELECDriver->namechannel0(index));
	}
}

void DAQParamController::updateNameChannel1(int32 index) {
	if (m_ELECDriver->boardtype(index) == stereo_elec::FEBOARD) {
		namechannel.set(m_ELECDriver->feindex.at(index) * stereo_elec::NBFECHANNEL + 1, m_ELECDriver->namechannel1(index));
	}
}
void DAQParamController::updateNameChannel2(int32 index) {
	if (m_ELECDriver->boardtype(index) == stereo_elec::FEBOARD) {
		namechannel.set(m_ELECDriver->feindex.at(index) * stereo_elec::NBFECHANNEL + 2, m_ELECDriver->namechannel2(index));
	}
}
void DAQParamController::updateNameChannel3(int32 index) {
	if (m_ELECDriver->boardtype(index) == stereo_elec::FEBOARD) {
		namechannel.set(m_ELECDriver->feindex.at(index) * stereo_elec::NBFECHANNEL + 3, m_ELECDriver->namechannel3(index));
	}
}
void DAQParamController::updateNameChannel4(int32 index) {
	if (m_ELECDriver->boardtype(index) == stereo_elec::FEBOARD) {
		namechannel.set(m_ELECDriver->feindex.at(index) * stereo_elec::NBFECHANNEL + 4, m_ELECDriver->namechannel4(index));
	}
}
void DAQParamController::updateNameChannel5(int32 index) {
	if (m_ELECDriver->boardtype(index) == stereo_elec::FEBOARD) {
		namechannel.set(m_ELECDriver->feindex.at(index) * stereo_elec::NBFECHANNEL + 5, m_ELECDriver->namechannel5(index));
	}
}
void DAQParamController::updateNameChannel6(int32 index) {
	if (m_ELECDriver->boardtype(index) == stereo_elec::FEBOARD) {
		namechannel.set(m_ELECDriver->feindex.at(index) * stereo_elec::NBFECHANNEL + 6, m_ELECDriver->namechannel6(index));
	}
}
void DAQParamController::updateNameChannel7(int32 index) {
	if (m_ELECDriver->boardtype(index) == stereo_elec::FEBOARD) {
		namechannel.set(m_ELECDriver->feindex.at(index) * stereo_elec::NBFECHANNEL + 7, m_ELECDriver->namechannel7(index));
	}
}
void DAQParamController::updateStartActivated() {
	m_HVDriver->locked=m_ELECDriver->startActivated();
}

void DAQParamController::updateLogString() {
	log(Level::s_Info) << m_ELECDriver->logstring() << endlog;
}


void DAQParamController::updateTitle() {
	//if(experimentData->title()!=""){
		cout << "Title " << experimentData->title() << endl;
		log(Level::s_Info) <<  experimentData->title()  << endlog;
		m_ELECDriver->DAQParams.title=experimentData->title();
	//}
}

void DAQParamController::updateSubtitle() {
	//if(experimentData->subtitle()!=""){
		cout << "subtitle " << experimentData->subtitle() << endl;
		log(Level::s_Info) <<  experimentData->subtitle()  << endlog;
		m_ELECDriver->DAQParams.subtitle=experimentData->subtitle();
	//}
}



/*
 * select channels
 */
void DAQParamController::select() {
	if (selectchoice() == "all") {
		for (int i = 0; i < ntotchannel(); i++) {
			isselected.set(i, true);
		}
	} else if (selectchoice() == "none") {
		for (int i = 0; i < ntotchannel(); i++) {
			isselected.set(i, false);
		}
	} else if (selectchoice() == "slot") {
		for (int i = 0; i < ntotchannel(); i++) {
			if (islot(i) == selectslot())
				isselected.set(i, true);
			//else isselected.set(i,false);
		}
	}else if(selectchoice()=="name"){
		for(int i=0;i<ntotchannel();i++){
			if(namechannel(i).find(selectstring())!=std::string::npos) isselected.set(i,true);
			//else isselected.set(i,false);
		}
	}

}

void DAQParamController::set() {
	for (int i = 0; i < ntotchannel(); i++) {
		if (isselected(i) == true) {
			if (setparam() == "Gain20") {
				if (setvalue() == "true")
					gain20.set(i, true);
				else
					gain20.set(i, false);
			} else if (setparam() == "TrigThres")
				trigthres.set(i, strtol(setvalue().c_str(), NULL, 10));
			else if (setparam() == "CFDThres")
				cfdthres.set(i, strtol(setvalue().c_str(), NULL, 10));
		}
	}
}

/*
 * resize
 */
void DAQParamController::resize() throw (CannotOpenFile, SlicesFileError, TooManySlices, SliceTooBig) {
	m_Detector->setDataSize(128, 1);
}

/*
 * writeParams
 */
void DAQParamController::writeParams() {
	commandProgression = 0;
	commandProgression = 100;
}

/*
 * synchronizeRead
 */
void DAQParamController::synchronizeRead() {
	commandProgression = 0;
	commandProgression = 100;
}

}

