/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef STEREOELECDRIVER_H
#define STEREOELECDRIVER_H

#include <Driver.h>
#include "StereoElecDef.h"
#include "STDAQParams.h"
#include "controllers/common/acquisition/mode/NoTofAcquisitionCommon.h"
#include "controllers/common/acquisition/gate/TimeGateCommon.h"
#include "controllers/common/acquisition/detector/DetectorCommon.h"

namespace stereo_elec {

/*!
 * \class StereoElecDriver
 * \brief Main class for the StereoElecDriver for the stereo electronics
 *
 * \par
 * Class for StereoElecDriver. Implements DeviceDriver methods.
 * \par
 * The StereoElecDriver device manages the global functionality of the stereo electronics.
 */

class StereoElecDriver: public virtual DeviceDriver, public acquisition::NoTofAcquisitionCommon, public acquisition::TimeGateCommon, public acquisition::DetectorCommon {

	friend class RealStereoElecDriver;
	friend class PerfectStereoElecDriver;

public:

	//! Driver type value
	static const std::string TYPE;

	/*!
	 * \brief Constructor
	 * \param[in] name the name of the device driver
	 */
	StereoElecDriver(const std::string& name);

	/*!
	 * \brief Destructor
	 */
	virtual ~StereoElecDriver();

	/*!
	 * \brief Method called for executing a command
	 *
	 * \param[in] command the command to apply on the controller
	 */
	void execute(const string& aCommand);

	virtual void refreshNFEBoardProperty(int32 value) throw (CannotSetValue);
	virtual void refreshNLEDBoardProperty(int32 value) throw (CannotSetValue);
	virtual void refreshBoardTypeProperty(int32 index, const std::string& value) throw (CannotSetValue);

	Property<std::string> mchipaddress;
	Property<std::string> triggermacaddress;
	Property<std::string> triggeripaddress;
	Property<int32> nslot;
	Property<int32> nfeboard;
	Property<int32> nledboard;
	DynamicProperty<std::string> boardtype;
	DynamicProperty<std::string> boardmacaddress;
	DynamicProperty<std::string> boardipaddress;
	DynamicProperty<std::string> namechannel0;
	DynamicProperty<std::string> namechannel1;
	DynamicProperty<std::string> namechannel2;
	DynamicProperty<std::string> namechannel3;
	DynamicProperty<std::string> namechannel4;
	DynamicProperty<std::string> namechannel5;
	DynamicProperty<std::string> namechannel6;
	DynamicProperty<std::string> namechannel7;
	DynamicProperty<int32> slotnumber;

	Property<bool> isconfigured;

	// Properties for MTCA monitoring
	// FE, size NBFEMAX
	DynamicProperty<float64> fe_pos5v; /**< Front-end positive power supply. */
	DynamicProperty<float64> fe_neg5v; /**< Front-end negative power supply. */
	DynamicProperty<float64> fe_adc3v3a; /**< ADC power supply. */
	DynamicProperty<float64> fe_adc1v8a;/**< ADC power supply. */
	DynamicProperty<float64> fe_tadc03;/**< Temperature probe close to ADC0-3.*/
	DynamicProperty<float64> fe_tadc47;/**< Temperature probe close to ADC4-7.*/
	// LED, size NBLEDMAX
	DynamicProperty<float64> led_dac3v3; /**< DAC setting power supply */
	DynamicProperty<float64> led_amp24v; /**< Amplifier supply. */

	// Properties for rates
	DynamicProperty<float64> rate0;
	DynamicProperty<float64> rate1;
	DynamicProperty<float64> rate2;
	DynamicProperty<float64> rate3;
	DynamicProperty<float64> rate4;
	DynamicProperty<float64> rate5;
	DynamicProperty<float64> rate6;
	DynamicProperty<float64> rate7;

	Property<int32> t1accepted;
	Property<int32> t1rejected;
	Property<int32> fiforejected;
	Property<int32> t2accepted;
	Property<string> runname;

	Property<std::string> logstring;

	STDAQParams DAQParams;
	vector<int32> feslots;  // size nfeboard
	vector<int32> ledslots; // size nledboard
	vector<int32> feindex;  // size nslot
	vector<int32> ledindex; // size nslot

	void computeFEIndex();
	void computeLEDIndex();

	//void slotsComputation();

	int32 nbevent;

	bool started;


private:

	static const std::string READ_CONFIG_COMMAND;
	static const std::string DAQ_CONFIG_COMMAND;
	static const std::string READ_MONITORING_COMMAND;

	bool m_SynchroniseReadActivated;
	bool m_SynchroniseReadTerminated;
	bool m_StopActivated;

	void calculateProgression();

	/*!
	 * \brief compute global gate status
	 * \param[in] value the driver status
	 */
	virtual void computeGateStatus(int32 value);

	/*!
	 * \brief compute global acquisition status
	 * \param[in] value the driver status
	 */
	virtual void computeAcquisitionStatus(int32 value);

	/*!
	 * \brief compute global detector status
	 * \param[in] value the driver status
	 */
	virtual void computeDetectorStatus(int32 value);

};

}
#endif //STEREOELECDRIVER_H
